function [xopt,fopt,lopt,par] = ...
                  artrust(matA,a,s,par)
%<html> <pre> DATE=Feb 4, 2000
%NEWTRUST Finds an approximate solution to the trust region subproblem.
%
%  ARTRUST solves: min      f(x) = x'Ax - 2a'x 
%                 such that        s^2-x'x >=0    (Lagr. mult. lopt <=0)
%%  which is the trust region subproblem. This can be used within
%%     a trust region algorithm for min f(x).
%
% The algorithm returns an approximation xopt of the true optimum x*;
%   and a corresponding value fopt=f(xopt) so that:
%         norm(xopt) <= (1+normtol)s
%   and two constants mulow,muup so that
%   mulow <= f(x*) <= muup,   mulow <= fopt <= muup,  AND
%  |f(x*) - fopt| <=  |muup-mulow| <= |2*dgaptol*muup| <= |2*dgaptol*fopt|
%  So that f(x*) >= (1-2*dgaptol)*fopt
%
%  [xopt,fopt] = artrust(A,a,s) returns the approximations xopt and
%    fopt for the data A,a,s
%
%  [xopt,fopt,lopt,par] = artrust(A,a,s,par) returns the approximations
%    fopt and xopt as well as an approximation lopt to the optimal
%    Lagrange multiplier. The structure par contains parameters on
%    input and output (described below).
%
% %%%%%%%%%%%%%%%%%%%%%% INPUT VARIABLES:
% A   The first input argument is either a real square
%     symmetric matrix (which can be full or sparse) or a
%     string containing the name of an M-file which applies a linear
%     operator to the columns of a given matrix.
%     For example, newtrust('fft',...) is much faster than EIGS(F,...)
%     where F is the explicit FFT matrix. This function is passed
%     to a sparse eigenvalue routine.
%
% a   is an n-vector which is the linear term of the quadratic.
% s   is the radius of the trust region.
%
% par is a structure with several optional inputs.
%     par.zerotol
%     par.mulow, par.muup
%%    ?????? 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% optional files needed: Afun.m,Dfun.m, and this file artrust.m  ???
%%%        option file conj_grad.m, see commented out part below  ???
%%
% BEFORE USING NEWTRUST, THE FOLLOWING INSTRUCTION MUST BE ENTERED
%           > global A D counter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%<head><title>Trust Region Subroutine Algorithm</title>  <center><H1>Trust Region Subroutine Algorithm</H1> <H2> Algorithm with Links to <a href="docs.html">Documentation</A><br> <br> Charles Fortin and Henry Wolkowicz<br>  Dept. Comb. & Opt., Univ. of Waterloo
%</H2><H3>This file is available with URL<br> <A href="http://orion.math.uwaterloo.ca/~hwolkowi/henry/software/trustreg.d/newtrust.html">http://orion.math.uwaterloo.ca/~hwolkowi/henry/software/trustreg.d/newtrust.html</A>
%<hr></head><BODY> In Documentation: <a href="docs.html"> Complete Documentation File</a>,<a href="docs.html#initvrbls"> Variable Definitions</a>, <br> In (this file) Algorithm: <a href="newtrust.html#tolerances"> Tolerances</a>, <a href="newtrust.html#initializ"> Initialization</a>,<a href="newtrust.html#while"> While Loop</a>,<a href="newtrust.html#endwhile"> End of While Loop</a>, </center>
%<!-- begin commented out 
%end commented out --!>
%<br><br> 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%
%
global A D counter



if ~ischar(matA)
   [n,n] = size(A);
   if any(size(A) ~= n)
      error('Matrix A must be a square matrix or a string.')
      return
   end
end
n=length(a);
%%%%%%%
%if s < zerotol,   %????too small???? what is too small here???
%   xopt=0;
%   fopt=0;
%   lopt=0;
%end                  %?????????????
%%%%%%%
if n==1,
  if A > 0 & abs(xopt) <= s,
     xopt=a/A;
     lopt=0;
  else
     xopt=sign(a)*s;
     lopt=A+b/xopt;
  end
     fopt= A*xopt^2-2*a*xopt;
  return
end
%%%%%%%%%%
     



%
%<A name="tolerances">
%%%%%%%%%%%%%%%%%%%%%% <A href="docs.html#tolerances">TOLERANCES</A>%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Back to <a href="newtrust.html">Top of algorithm page.</A>
% Tolerances - these can and should be changed by the user.
%
if nargin < 4,
   par.gaptol=1e-9;   % default tolerance
   dgaptol=par.gaptol;    % duality gap tolerance
else
   dgaptol=par.gaptol;     % duality gap tolerance
end
intol= dgaptol;
normtol  =intol;         % allowable error in feasibility for stopping
zerotol=intol/log(n^100); % tolerance for zero for y(1) for hard case
                        % since y is norm 1, tolerance decreases with dim.
kzerotol=zerotol;       % tolerance for zero for k(t)
hzerotol=1e-4;      % hard case detection  ???
iterbnd=30;             % upper bound on iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%END TOLERANCES%%%%%%%%%%%%
% 
%<A name="initializ">
%%%%%%%%%%%%%%%%%%%%%%%%%%%INITIALIZATION%%%%%%%%%%%%%%
% Back to <a href="newtrust.html">Top of algorithm page.</A>
t0=cputime;             % saved to get total cputime usage
counter = 0;            % counts matrix mults
a=full(a);              %  set to full to avoid conflicts
na=norm(a);             % vector norm
if na    <   zerotol,
   disp('WARNING: linear part -a- is approximately 0');
end

%%%First eigenvalue calculation
OPTIONS.tol=1e-13;   % convergence tolerance
OPTIONS.issym = 1; % because we know the matrices A and D are symmetric
OPTIONS.isreal = 1; % A is real
OPTIONS.disp = 0; % no display of the output for eigs.m
v=full(sum(A))';v=-v/norm(v);  % good/cheap normalized estimate ???
                %i.e. largest eig of -A using v=-Ae, e=ones
OPTIONS.v0 =v ; % initial eigenvector estimate for sparse A
%%%a and t are passed to Afun ????
tarn10=cputime; % keep for the first eig calc time
if ischar(matA)
   [va,lambda,flageigs]=areigs(matA,n,1,'SA',OPTIONS); % computes smallest
else
   [va,lambda,flageigs]=areigs(A,1,'SA',OPTIONS); % computes smallest
end
tarn1f=cputime-tarn10;   % save initial eig calc cputime for statistics
disp(['elapsed time for first eig calc:  ',num2str(tarn1f)]);
if (flageigs>0),
      disp('WARNING INITIAL EIGENVALUE CALCULATION BAD');
      disp(['condition est. of eigs:  ',num2str(condest(A))]);
end
%           %eigenvalue (lambda) and corresponding eigenvector (va) for A
avnorm=norm(A*va-lambda*va);
%if avnorm > zerotol*(abs(lambda)+1),   %  ???zerotol???check???
%      disp(['USING eig, output from eigs norm error ',num2str(avnorm)]);
%      [V,Lam]=eig(full(A));
%      lambda=min(diag(Lam));
%      indx=find(lambda==diag(Lam) );va=V(:,indx);va=va/norm(va);
%      avnorm=norm(A*va-lambda*va);
%      disp(['USING eig new norm error is  ',num2str(avnorm)]);
%end

lambdamin = lambda; % save the min eigenvalue of A
vaa=va'*a;
% ensure va'*a is not negative
if (vaa < 0)
   va = -va;vaa=-vaa;
end
y=[1;va];y=y/norm(y);  % initial normalized estimate eigenv. D(t^*)
flag = 0; %flag will change value if the almost hard case (case 2) occurs 

alow=lambda;aup=lambda;
% lower estimate on mu*
if lambda<0
   mulow = alow*s^2-2*na*s;  % lower estimate for optimal value
else
   % mulow = -2*na*s;  %  replaced by min on boundary or interior
      % since interior opt value is a'invAa-2a'invAa=-a'invAa
   mulow = min(alow*s^2-2*na*s,-na*s);
end
% upper estimate on mu*
if lambda > 0
  if (( (vaa)/(lambda )  <=  s)),   %  optimum may be on boundary or interior
     testmu = s^2*lambda - 2*s*na;
     muup  = -(vaa)^2/lambda;
     if muup < testmu,   %  apply conj grad - opt interior
       a=sparse(a);
       %xopt = conj_grad((vaa/lambda)*va,a); 
            % apply conjugate gradient method to  find the optimum.
       xopt=A\a;     % use matlab function for now or uncomment conj_grad

       fopt  = -a'*xopt;   % optimal value since A xopt = a
       lopt = 0; 
       topt = 99999;
       muup=99999; mulow=99999;iter=0;
       gap=[topt topt
            lopt lopt
            topt topt
               ];
       t0f=cputime-t0;
       flag = 2; % To know answer is the unconstrained minimum
       disp(['elapsed time: ', num2str(t0f)]);
       disp(['Problem solved using conjugate gradients  ']);
       relopt=-norm(  (A-lopt*eye(size(A,1)))*xopt-a)/fopt;
       %rel opt cond. norm grad of Lagrangian / value of Lagrang. at opt.
       format long e
       disp(['relative opt. condition:  ', num2str(relopt)])
       return;    %OPTIMALITY - PROBLEM IS SOLVED
     end
     bestxf = ((vaa)/lambda)*va; % vaa is >=0 now
  else
     muup = s^2*lambda - 2*s*(vaa);  % <=0 by above if 
     bestxf = s*va; 
  end
else
  [muup,indice] = min([0,lambda*s^2-2*s*(vaa)]);
  if indice == 1
    bestxf = zeros(size(a,1),1);
  else
    bestxf = s*va;
  end
end
bestfxf=muup; % current best opt value corresp to bestxf
              % bestxf is feasible (possibly interior)
mubnds=[mulow muup];    % keep history of bounds - latest at top
                        % This is only for debugging help.
gap=[mulow muup];       % duality gap - gets updated when a bound changes

if (lambda > 0)
  low = min(alow - na/s,-na*s);   %lower bound on t
else
  low = alow - na/s;
end
high = aup + na*s;      % upper bound on t
newlow=low;newhigh=high;   %new points found by vertical cuts
vertflag=0;             % to prevent 2 vertical cuts in a row

ss1=s^2+1; 
D = [ 0 -a' 
     -a   A];           % augmented matrix
ystar = 1 / sqrt( s*s +1);      % this is value we are after for y0(t)
iter=0;         % iteration count
b6 = '      ';              % define 6 blanks

% ptgood, ptbad variables: used for saving interpolation data on the
%  good (easy) and bad (hard) side of maximum of k(t).
%  gy,by saves last y vector on good and bad side
% The latest data is put into row 1.
%   Only two rows are needed. The rest is saved for debugging help.
%    t, y(1), lambda,  valid-data-indicator ; is data saved.
ptbad=[];
ptgood=[];
gy=[];by=[];    % The latest vectors y from the good and bad side resp.
side=0;        % indicator for consecutive t found 
                %  -i for i bad sides and +i for i  good sides
tslopes=[];    % Save slopes of kt on the [bad good] sides - for
%              debugging and for case where kt is almost piecewise linear.
kts=[];      % saving kt bounds and t value - for debugging
t=99999;indexg=0;indexb=0;  % initialize in case of 0 iterations
tind=[];     % follow trail of t changes - for debugging
%
disp('iter ----- t  ----  rel dual gap -- rel. primal opt cond');
%txt=[int2str(iter),b6,num2str(t),b6,num2str(dgap),b6,num2str(dbestgap)];
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%NOTES on ALGORITHM%%%%%%%%%%%%%%
%   Following are the functions of interest:
%    h(lambda)  =  s^2lambda -a'(A-lambda I)^{-1} a;      maximize - concave
%    hp(lambda)  = s^2 -a'(A-lambda I)^{-2} a;solve = 0 - concave, nonincreasing
%    phi(lambda)  = 1/s-1/||(A-lambda I)^{-1} a||;solve = 0 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    k(t)  =  (s^2+1)lambda -t         maximize - concave
%    k'(t) = (s^2+1)y0^2(t) -1         solve = 0  - same sign as f(t)
%    psi(t)  = -1/y0(t) + sqrt(s^2+1)    solve = 0 - concave, nonincreasing
%    y0(t) = first component of y normalized   - nonincreasing, 1to0
%    lambda=lambda(t)    concave function
%
% The algorithm is based on maximizing k(t).
% We interpolate on psi(t) whenever possible.
%
% The general iteration interpolates psi(t) whenever
%   possible. Duality is used to calculate bounds, i.e. the function k(t) is
%   used when problems with the hard case occur.

% Initialize the stopping criteria
t1=abs( 1-y(1)^2*(1+s^2))>( (2*normtol+normtol^2) *y(1)^2*s^2 );
dbestgap= (bestfxf-mulow)/(abs(bestfxf)+1);
t2= dbestgap   > 2*dgaptol;
dgap=  abs(muup-mulow)/(abs(muup)+1) ;
t3=  (muup-mulow) >= -2*dgaptol*muup;  
           %t3= dgap >  dgaptol; % Duality gap criteria    - old way
t4=iter <=iterbnd; % iteration bound
t5=abs(high-low)/(abs(high)+1) > zerotol; %interval of uncertainty for t
%%%%%%%%%%%%%%%%%%%END INITIALIZATION%%%%%%%%%%%%%
%%
%<A name="while">
%%%%%%%%%%%%%%%%%%%%%%%WHILE LOOP%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%Back to <a href="newtrust.html">Top of algorithm page.</A>, 
%      <a href="newtrust.html#findt"><b>PART 1.</b> Find New Point t</A>,
%              <a href="newtrust.html#vertical">Vertical Cut</A>,
%              <a href="newtrust.html#triangle">Triangle Interpolation</A>,
%              <a href="newtrust.html#goodside">Good Side Interpol.</A>,
%              <a href="newtrust.html#badside">Bad Side Interpol.</A>,
%      <a href="newtrust.html#updates"><b>PART 2.</b> Updating Begins.</A>,
%             <a href="newtrust.html#lambdag0">Lambda has wrong sign</A>,
%             <a href="newtrust.html#badsideu">Bad Side Update</A>,
%             <a href="newtrust.html#goodsideu">Good Side Update</A>,
%       <a href="newtrust.html#endwhile"><b>PART 3.</b>Tolerance Updates, End of While Loop, Start of Output</A>,
%
%% check feasibility and iteration count and duality gap
while ( (t1&t2) | t3) & (t4 & t5),  % start of while%<ul> 
  iter=iter+1; % one more iteration is done
  indexg=size(ptgood,1); % number of points on the good side
  indexb=size(ptbad,1); % number of points on the bad side
  %<A name="findt">
  %%%%%%%%%  FIND a new point t.
  newlow=max(newlow,low);newhigh=min(newhigh,high);
  if newlow > low,
     t=newlow;
     tind=[tind -2];    % indicator for t value - for debugging
  elseif newhigh < high,
     t=newhigh;
     tind=[tind -1];
  else
     t=(max(newlow,low)+min(newhigh,high))/2;      % midpoint default
     tind=[tind 1];
  end

  if indexb > 0 & indexg > 0,         %   <ul>
     %  The last point on right and left that has eigenvalue data
     % always do triangle interpol and vertical cut if possible
     ktl=ss1*ptbad(1,3)-ptbad(1,1); 
     kth=ss1*ptgood(1,3)-ptgood(1,1);
     tlslope=ss1*ptbad(1,2)^2-1;
     thslope=ss1*ptgood(1,2)^2-1;

     %
     %<A name="triangle">
     %%%%%%%%%%%%%%%%%%%%%%%TRIANGLE INTERPOLATION%%%%%%%%%%%
     % Back to <a href="newtrust.html">Top of algorithm page.</A>

     % Always find interpolant using triangle-with associated upper bound.
     tslopes=[tlslope thslope
               tslopes];         % for debugging purposes - possible use
                                % when both sides are almost linear
     mrhs=[  (tlslope*(-ptbad(1,1))+ktl)
               (thslope*(-ptgood(1,1))+kth)     ]; 
                                    % rhs of interpolation
     ktt=[ 1  (-tlslope)
           1  (-thslope)];
     kttans=ktt\mrhs; % find intersection of the two tangent lines at
            % the last bad and good side points. The point of
            % intersection is (kttans(2),kttans(1)). Hence, kttans(1)
            % is an upper bound on mu^* and kttans(2) is an approximation
            % for t^*
     ktup=kttans(1);
   %%%%%%%?????testing the next if ????!!!
      %%%???change!!!!in case near hard case - this is always too big
        %%%???a step away from good side?????
     if ptgood(1,2) >zerotol,   % avoided hard case here - try 
         tt=kttans(2);   % This is alternate interpol point using triangle
     else
         tt=.9*newhigh+.1*tt;
     end


     if (ktup <=muup) & (ktup >= mulow),
         muup=ktup;           % new upper bound on optimal value
     end
     if max(newlow,low)<=tt & tt <=min(newhigh,high);
         t=tt;
         tind=[tind 2];
     end
     %  %%%%%%%%%%%%%%%%%%%%End of triangle for new interpolant   


     %
     %<A name="vertical">
     %%%%%%%%%%%%%%%%%%%%%%%VERTICAL CUT%%%%%%%%%%%
     % Back to <a href="newtrust.html">Top of algorithm page.</A>
     %
   if ptgood(1,2) >zerotol,   % avoided hard case here - try 
                             %%% smaller values for (was) hzerotol!!!
    if vertflag==1,                 % <ul>
     vertflag=0;                 % </ul>
    else                 % <ul>
     vertflag=1;          % prevents 2 vertical cuts in a row
             %% errors can occur if eigenvalue not calculated properly
              %% or in hard case - here and below!!!
      if kth > (1+kzerotol)*ktl;  % opt cond for k(t) - check???<ul>
        tempnewlow=low + (kth-ktl)/tlslope;
        if newlow <= tempnewlow & tempnewlow <= newhigh,       % <ul>
           newlow=tempnewlow;
           t=newlow;
           tind=[tind -2];
        else
           disp('WARNING newlow not in interval ?????')
           %%????this is not explained satisf. why??? roundoff
           %%%that important???
           newlow=low;            % </ul>
        end            % </ul>
      elseif ktl > (1+kzerotol)*kth;  %opt cond for k(t)-check?<ul>
        tempnewhigh=high + (ktl-kth)/thslope;
        if newlow <= tempnewhigh & tempnewhigh <= newhigh,  %<ul>
           newhigh=tempnewhigh;
           t=newhigh;           % </ul>
           tind=[tind -1];
        else           % <ul>
           disp('WARNING newhigh not in interval ?????')
        end                           %</ul>
      else                             %<ul>
        disp('two values of k up to tolerance found');
                            % arrange proper output still????RETRN!!!</ul>
     end                 % </ul>
    end                  %end vertical flag check
   end          %  end of hard case check above vert flag -  temp
%%%%%%%%%%%%%%%%%%%%%%%end vertical cut%%%%%%%%%%%  </ul>

  end     % end of if indexb > 0 and indexg > 0 

%%%%%%%%%%%%%%%%%%%%%%%%%
%<A name="goodside">%%%%%%%%% INVERSE INTERPOLATION - good side
%????how can these two lines not be executed???even though the if is executed???
 if side>0,   % last iterate was on the GOOD SIDE   <ul>

  if  sum(ptgood(:,4)) >=2, %two t-iterates with data good side  <ul>
     tint=ptgood(1:2,1);
     yint=ptgood(1:2,2);
     f1=-1/yint(1)   + 1/ystar;
     f2=-1/yint(2)   + 1/ystar;
     slope=(tint(1)-tint(2))/(f1-f2);
     tt=tint(1) - f1* slope;    % linear inverse interpolation
disp('before NaN')
keyboard
     if newlow <=tt&tt<=newhigh,     % roundoff error problem check  <ul>
       %% We now decide whether to chose the new point
        t=tt;
        tind=[tind 3];     % indicator for t value - for debugging</ul>
     end
     if side>4&(min(ptgood(1:2,2)) < (10^(side-1)*zerotol) ), %<ul>
        %too many iterates on good side and it is near bad case
        t=.95*newlow+.05*newhigh;
                        %????use weights from k' if indexb>0?????
        tind=[tind 4];     % indicator for t value - for debugging </ul>
     end               % </ul>
  elseif sum(ptgood(:,4)) >=1 & indexb>0 >= 1,
     if ptgood(1,2)>zerotol & tind(length(tind)) >= 0,    
              % not near hard case  and not vertical cut
       % interpolate with one point from each side if not hard case
       tintg=ptgood(1,1);
       tintb=ptbad(1,1);
       psig=-1/ptgood(1,2)+sqrt(ss1);
       psib=-1/ptbad(1,2)+sqrt(ss1);
       tt=(tintg*psib-tintb*psig)/(psib-psig); 
       if newlow <= tt & tt <= newhigh,
          t=tt;
          tind=[tind 31];
       end
     else
       %use spline on k(t) - to do still
          tind=[tind 32];
     end
  end   % end of if sum(ptgood) >=2 else >=1

  %%%%%%%%%%%%%%%%%%%%%%%%%
  %<A name="badside">             </ul>
 elseif side<0,   % last iterate was on the BAD SIDE  <ul>
  if sum(ptbad(:,4)) >=2,
     %INVERSE INTERPOLATION
     % t(psi) inverse function of psi(t), then we linearly interpolate
     % value t(0), which is t^*, using two points from the bad side. 
     tint=ptbad(1:2,1);
     yint=ptbad(1:2,2);
     f1=-1/yint(1)   + 1/ystar;
     f2=-1/yint(2)   + 1/ystar;
     % If f1=f2, then t value repeated by e.g. t=low in triangle.
     % This indicates that the optimal t is very close to t=low.
     if f2>f1,   % correct monotonicity occurred
      slope=(tint(1)-tint(2))/(f1-f2);
      tt=tint(1) - f1* slope;    % linear inverse interpolation
        if newlow<=tt&tt<(newhigh-zerotol), 
                  % Use the new point - it is not too large
                  % This should go to good side
          t=tt;
          tind=[tind 5];     % indicator for t value - for debugging
        elseif side<-4,   % need a point on good side desperately
          % was following two lines - need to speed this up using a 
          %   CONCAVE spline  for k(t) in hard case!!!!
          t=.95*high + .05*low;
          tind=[tind 7];     % indicator for t value - for debugging
        end
     end                   % </ul>
  elseif  indexg>0 & ptgood(1,2)>hzerotol & tind(length(tind)) >= 0,   
               % not near hard case and not vertical cut
                  % this interpolation makes no sense if near hard case
       % interpolate with one point from each side if not hard case
       tintg=ptgood(1,1);
       tintb=ptbad(1,1);
       psig=-1/ptgood(1,2)+sqrt(ss1);
       psib=-1/ptbad(1,2)+sqrt(ss1);
       tt=(tintg*psib-tintb*psig)/(psib-psig); 
       if newlow <= tt & tt <= newhigh,
          t=tt;
          tind=[tind 31];
          if side<-3,   % need a point on good side desperately
             % was following two lines - need to speed this up using a 
             %   CONCAVE spline  for k(t) in hard case!!!!
             t=.95*high + .05*low;
             tind=[tind 7];     % indicator for t value - for debugging
          end
       else
            %In case near hard case above !!!!
            %  %use spline on k(t) - to do still
                  tind=[tind 32];
       end
  end    %  end for sum(ptbad) >=2 and >=1     </ul>
 end    % end for side check
disp(['side: ', num2str(side)]);
disp([sprintf('trail:'), sprintf('%d,%d,',tind) ])



%<A name="updates">  
%%%%%%%%%%%UPDATES%%%%%%%%%%%

%  A new point t has been found. Start updates.
%
% Recall t is the most recent update for t^*

D( 1, 1) = t;

tarn20=cputime;

OPTIONS.tol=1e-13;   % convergence tolerance
OPTIONS.v0 = y;       %  default
if tind(length(tind))==3,  % finished interpol on good side
      OPTIONS.v0 = gy;
elseif tind(length(tind))==31,  % finished interpol using both sides
      OPTIONS.v0 = by;
elseif tind(length(tind))==7 & indexg > 0, %  very close to high
      OPTIONS.v0 = gy;
elseif tind(length(tind))==-2, %  close to low
      OPTIONS.v0 = by;
elseif tind(length(tind))==-1, %  close to high
      OPTIONS.v0 = gy;
elseif tind(length(tind))==4 & indexb > 0, %  very close to newlow
      OPTIONS.v0 = by;
elseif tind(length(tind))==5 & indexg > 0, %  go to good side
      OPTIONS.v0 = gy;
elseif (t-low) < ( (high-t) - 100*zerotol ) & indexb > 0,
      OPTIONS.v0 = by;
elseif (t-low) > (  (high-t) + 100*zerotol )& indexg > 0,
      OPTIONS.v0 = gy;
end
if ischar(matA)
   [va,lambda,flageigs]=areigs('Dfun',n+1,1,'SA',OPTIONS); % computes smallest
else
   [va,lambda,flageigs]=areigs(D,1,'SA',OPTIONS); % computes smallest
end
%[y,lambda,flageigs] = eigs('Dfun',n+1,1,'SR',OPTIONS); 
tarn2f=cputime-tarn20;  % save initial Lanczos call cputime for statistics
disp(['tind(last) and elapsed time for second eig calc:  ', ...
            num2str(tind(length(tind))),' ,  ',num2str(tarn2f)]);
if (flageigs>0),
      disp('flag WARNING from eigs EIGENVALUE CALCULATION BAD');
      disp(['condition est. of eigs:  ',num2str(condest(D))]);
end
%                   % y and lambda are the new estimates
%                   % for the eigenvector of D(t^*) and for lambda^*
%dynorm=norm(D*y-lambda*y);
%if dynorm > zerotol*(abs(lambda)+1),   %  ???zerotol???check???
%      disp(['USING eig, output from eigs norm error ',num2str(dynorm)]);
%      [V,Lam]=eig(full(D));
%      lambda=min(diag(Lam));
%      indx=find(lambda==diag(Lam) );y=V(:,indx);y=y/norm(y);
%      dynorm=norm(D*y-lambda*y);
%      disp(['USING eig new norm error is  ',num2str(dynorm)]);
%end


if y(1)<0,  % ensure the sign of the first component is positive
  y=-y;
end

%<A name="lambdag0">  
%  multiplier sign check - to update lower bounds
if lambda > 0,   % A is known to be pos def - multiplier has wrong sign

   if y(1) >= ystar,
   % means y(2:n+1)/y(1) is the optimum solution for all points outside
   % trust region of radius (1-y(1)^2)/y(1)^2 or on its boundary. If this 
   % radius is less than s^2, as is case here since this is equivalent to
   % y(1) >= ystar, then the optimum we are looking for is inside
   % the trust region and A is positive semidefinite. 
   % We will then apply a conjugate
   % gradient method to find the optimum. 
   % Hence, y(2:n+1)/y(1) is feasible so optimal with lambda=0.

       a=sparse(a);
       %xopt = conj_grad(y(2:n+1)/y(1),a); 
       %     % apply conjugate gradient method to  find the optimum.
       xopt=A\a;     % use the matlab function for now or uncomment conj_grad
       gap  = -a'*xopt;   % optimal value since A xopt = a
       fopt=gap;
        lopt = 0; 
        topt = 99999;
        gap=[topt topt
             lopt lopt
             topt topt
               ];
        t0f=cputime-t0;
        flag = 2; % To know answer is the unconstrained minimum
        disp(['elapsed time: ', num2str(t0f)]);
        disp(['Problem solved using conjugate gradients  ']);
        relopt=-norm(  (A-lopt*eye(size(A,1)))*xopt-a)/fopt;
        %rel opt cond. norm grad of Lagrangian / value of Lagrang. at opt.
        format long e
        disp(['relative opt. condition:  ', num2str(relopt)])
        return;    %OPTIMALITY - PROBLEM IS SOLVED
    end
else     % the sign of the multiplier is correct
         %   so apply duality theory
    kt=ss1*lambda-t;         % value of k(t), dual functional. This gives
                               % a lower bound on mu^*
    kts=[t lambda kt
            kts];
    if (kt>mulow)&(kt<=muup),
      mulow=max(mulow,kt);           % new lower bound on optimal value
    end
%
end        % end of if lambda > 0 loop
%%  end multiplier sign check

%
%<A name="badsideu">  
%%%%%%%%%%%%%Bad side update
if y(1) >= ystar,        % bad side
%%%%???? when y(1) very small and ystar very small, then error can
%%% occur about knowing which side - roundoff problem can occur
%%%   here???? analysis needed.
     if t==newhigh;      %% roundoff error problem here ????why???
                        %%%newhigh was on wrong side
        newhigh=high;
     end
     low = t;           % new lower bound on t^*
     ptbad=[t y(1) lambda  1
           ptbad];       % Keep points for first interpolation.
    side=min(-1,side-1);  % consecutive bad sides
    by=y;
%%
%%
%%%%%%%%%%%%%
%% Begin bound for PSDP from bad side - approx. negative curvature
%  direction and move to the boundary.
   if indexg>0,
    if  min(ptgood(:,3)) <= 0,   % multiplier has correct sign on good side
     normx2=(1-y(1)^2)/(y(1)^2); % square of norm of current soln.
     xf=y(2:(n+1))/y(1);   % current soln. inside trust region

% In the hard case (case 2), we use MoSo formula page 558 - however
% with negative eigenvalue and eigenvector, to move to take a step to
% the boundary. In the easy case and hard case (case 1), we take a step
% to the boundary by taking the solution between the line joining the 
% xf and the x-point - gy(2:n+1)/gy(1,1) -  outside the trust region
% associate with the last good side t-point.

	 if (gy(1) < zerotol)
		% This is the almost hard case (case 2)
		z = gy(2:n+1);
     	        normz=norm(z); % normz should be almost 1   

	 else
		% This is the easy case or almost hard case (case 1)
	 	z = gy(2:n+1)/gy(1,1)-by(2:n+1)/by(1,1);
	  	normz=norm(z);

	 end


     xyprod=(z'*xf)/normz;  % normalized inner product
     ndiff=s^2-normx2;
     sxy=sign(xyprod);
     if sxy==0,
        sxy=1;
     end
%%%??????check whether this is really the correct direction that 
%%% keeps the optimality conditions - seems perhaps not???
%% or perhaps because of roundoff??? z should be in null space
%%% of gradient of Lagrangian ????? or because it is on good side???
     tau=ndiff/(xyprod+sxy*sqrt(xyprod^2+ndiff) );
     xnew=xf+(tau/normz)*z;  % xnew is the point obtained by moving to the
                             % boundary
     fxnew=xnew'*A*xnew-2*xnew'*a; %should improve at point xf
   % get new upper bound using the feasible xnew - need to allow almost
      %  best xnew as the bestxf
     if fxnew<bestfxf,
        bestxf=xnew;
        bestfxf=fxnew;
     end
     if (fxnew<muup) & (fxnew >=mulow),
        muup=fxnew;
     end

    end      % end if <= 0 line mult. check
   end        %end of if indexg line
   %%%%%%%%%% end of PSDP bound
%     
%<A name="goodsideu">  
%%%%%%%%%%%%%Good side update
%%%%%%%%%%%%%%%%%%%
else                    %  current point is on good side
     if t==newlow;      % roundoff error problem correction - but 
        newlow=low; why did it occur????
     end
     high = t;       % we have new upper bound on t
     ptgood=[t y(1) lambda 1
             ptgood   ] ;
    side=max(1,side+1);  % consecutive good sides
    gy=y;
  if  ptgood(1,3) <= 0,   % current multiplier has correct sign
    if indexb > 0,
%% begin type 1 bound for PSDP from the good side
     normx2=(1-by(1)^2)/(by(1)^2);  % use last bad side point
     xf=by(2:(n+1))/by(1);

% In the hard case (case 2), we use MoSo formula page 558 - however
% with negative eigenvalue and eigenvector, to move to take a step to
% the boundary. In the easy case and hard case (case 1), we take a step
% to the boundary by taking the solution between the line joining the 
% xf and the x-point - gy(2:n+1)/gy(1,1) -  outside the trust region
% associate with the last good side t-point.

	 if (gy(1) < zerotol)
		% This is the almost hard case (case 2)
		z = gy(2:n+1);
                normz=norm(z); %normz  
	 else
		% This is the easy case or the almost hard case (case1)
	 	z = gy(2:n+1)/gy(1,1)-by(2:n+1)/by(1,1);
	  	normz=norm(z);
	 end


     xyprod=(z'*xf)/normz;  % normalized inner product
     ndiff=s^2-normx2;
     sxy=sign(xyprod);
     if sxy==0,
        sxy=1;
     end
     tau=ndiff/(xyprod+sxy*sqrt(xyprod^2+ndiff) );
     xnew=xf+(tau/normz)*z;  %
     fxnew=xnew'*A*xnew-2*xnew'*a;
   % get new upper bound using the feasible xnew - need to allow almost
      %  best xnew as the bestxf
     if fxnew<bestfxf,
        bestxf=xnew;
        bestfxf=fxnew;
     end
     if (fxnew<muup) & (fxnew >=mulow),
        muup=fxnew;
     end

    else      % else of indexb>0 line
% Need upper bound as well in case triangle was not used.
%
%% begin type 2 bound for PSDP
       xf=y(2:(n+1));    % norm is sqrt of 1-y_0^2
       xnew=(s/sqrt(1-y(1)^2))*xf; 
         % xnew is the point obtained by moving to the  boundary
       fxnew=xnew'*A*xnew-2*xnew'*a;
        % get new upper bound using the feasible xnew
       if (fxnew<muup) & (fxnew >=mulow),
          bestxf=xnew;
          bestfxf=fxnew;
          muup=fxnew;
       end
%% reached end type 2 bound for PSDP
   end     % ind of if indexb > 0 line
  end      % end if <= 0 line - multiplier sign check
end        % end of if bad side
%%%%%%%%%%%%
%% save new bounds and gap
if mulow > mubnds(1,1) | muup < mubnds(1,2),
  mubnds= [ mulow muup
      mubnds ];
  gap=[mulow muup];    % duality gap - gets updated when a bound changes
end

 %<A name="endwhile">
% update tolerances
t1=abs( 1-y(1)^2*(1+s^2))>( (2*normtol+normtol^2) *y(1)^2*s^2 );
                % use y(1)=ystar at optimum
%t1=abs( 1-y(1)^2*(1+s^2))>(normtol*y(1)^2*s^2 ); % old Feas criteria
dbestgap= (bestfxf-mulow)/(abs(bestfxf)+1);
t2= dbestgap   > 2*dgaptol; %??????why 2???change: rel dual opt cond
    % if t1 feasibility fails, but bestfxf works, then feasibility is fine
if muup==0,
   muupt=muup+1;
else
   muupt=muup;
end
reldgap=  abs(muup-mulow)/(abs(muupt)) ;
t3=  (muup-mulow) >= -2*dgaptol*muup;  
t4=iter <=iterbnd; % iteration bound criteria
t5=abs(high-low)/(abs(high)+1) > zerotol; % gap for t^* criteria
txt=[int2str(iter),b6,num2str(t),b6,num2str(reldgap),b6,num2str(dbestgap)];
disp( txt);
    %</ul>     end of while loop

end         % end of while for gap and iteration check
% %%%%%%%%%%%%%%%%%%%%%Back to <a href="newtrust.html">Top of algorithm page.</A>, 
%
%%%%%%%%%%%%%%%%%%%%%%%%%END GENERAL ITERATION%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%OUTPUT DATA%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fopt=(muup+mulow)/2;
if ~t2,           % bestxf is almost feasible
   xopt=bestxf;
   fopt=bestfxf;
else
   xopt = y(2:n+1)/y(1);
end
  topt=t;lopt=lambda;
  if indexg ==0,
       ptgood=[99999 99999 99999 0];
  end
  if indexb ==0,
       ptbad=[-99999 -99999 -99999 0];
  end
  t0f=cputime-t0;
  gap=[gap                    % lower and upper bound for mu^*
       ptbad(1,3) ptgood(1,3) % lower and upper bound for lambda^*
       low high               % lower and upper bound for t^*
       tarn1f t0f ];          % tarn1f is elapsed time for first eigs call
                              % t0f is total computation time

disp(['elapsed time: ', num2str(t0f)]); % t0f is total computation time
disp(['number of matrix-vector multiplications:  ', num2str(counter)])
disp(['number of iterations:  ', num2str(iter)])
relopt=-norm(  (A-lopt*eye(size(A,1)))*xopt-a)/fopt;
%rel opt cond. norm of grad of Lagrangian / value of Lagrang. at opt.
format long e
disp(['relative opt. condition:  ', num2str(relopt)])
disp(['interval of optimal value: [ ',num2str(mulow),' to ',...
             num2str(muup),' ]']);

disp('final results for  ');
disp('iter ----- t  ----  rel dual gap -- rel. dual opt cond');
txt=[int2str(iter),b6,num2str(t),b6,num2str(reldgap),b6,num2str(dbestgap)];
disp( txt);


if (abs(lambdamin - lopt)/(abs(lambdamin)+1) < 1e-2)
  flag = 1;
end

% <a href="newtrust.html">Top of algorithm page.</A> <br><img SRC="http://orion.math.uwaterloo.ca/~hwolkowi/images.d/back.xbm" ALT="[ DIR]" ><i><a href="http://orion.math.uwaterloo.ca/~hwolkowi/henry/software/trustreg.d">This directory on orion</a></i>  </BODY> </html>
